#!/bin/bash

if [[ $EUID -ne 0 ]]; then
  whiptail \
    --title "Невозможно запустить repka-config" \
    --msgbox "Утилита repka-config работает только из под пользователя root!\n\nДля запуска утилиты необходимо перейти в профиль root или воспользоваться аналогичной командой: sudo repka-config" \
    --ok-button "Ok"\
    10 70
  exit 1
fi

CONFIG_VERSION="v1.0.8_d09.01.25"
RADIOLIST_TIP="Подсказка: Для переключения между вариантами используйте клавиши 'Вверх' и 'Вниз', для выбора нажмите клавишу 'Пробел', для перехода к кнопкам 'Выбрать' и 'Назад' нажмите 'Tab'"

# Получение исходных значений
PINOUT=$(repka-control get-pinout)
CONSOLE_PARAM=$(repka-control get-console)
BOOT_MESSAGE=$(repka-control get-boot-message)
KERNEL_LOG_LEVEL=$(repka-control get-kernel-log-level)
ROOT_DEVICE=$(lsblk -no pkname $(findmnt -n -o SOURCE /))

CP1="$(repka-control get-custom-port 1)"
CP2="$(repka-control get-custom-port 2)"
CP3="$(repka-control get-custom-port 3)"
CP4="$(repka-control get-custom-port 4)"
CP5="$(repka-control get-custom-port 5)"
CP6="$(repka-control get-custom-port 6)"

# Сохранение исходных значений для последующего сравнения и использования в функции check_need_reboot
PINOUT_SAVE=$PINOUT
CONSOLE_PARAM_SAVE=$CONSOLE_PARAM
BOOT_MESSAGE_SAVE=$BOOT_MESSAGE
KERNEL_LOG_LEVEL_SAVE=$KERNEL_LOG_LEVEL
CP1_SAVE=$CP1
CP2_SAVE=$CP2
CP3_SAVE=$CP3
CP4_SAVE=$CP4
CP5_SAVE=$CP5
CP6_SAVE=$CP6

check_need_reboot() {
  IS_SAME=true

  [ "$PINOUT" != "$PINOUT_SAVE" ] && IS_SAME=false
  [ "$KERNEL_LOG_LEVEL" != "$KERNEL_LOG_LEVEL_SAVE" ] && IS_SAME=false
  [ "$CONSOLE_PARAM" != "$CONSOLE_PARAM_SAVE" ] && IS_SAME=false
  [ "$BOOT_MESSAGE" != "$BOOT_MESSAGE_SAVE" ] && IS_SAME=false

  if [ "$PINOUT" = "custom" ]; then
      [ "$CP1" != "$CP1_SAVE" ] && IS_SAME=false
      [ "$CP2" != "$CP2_SAVE" ] && IS_SAME=false
      [ "$CP3" != "$CP3_SAVE" ] && IS_SAME=false
      [ "$CP4" != "$CP4_SAVE" ] && IS_SAME=false
      [ "$CP5" != "$CP5_SAVE" ] && IS_SAME=false
      [ "$CP6" != "$CP6_SAVE" ] && IS_SAME=false
  fi

  if [ "$IS_SAME" = false ]; then
    if (
          whiptail \
            --title "Необходима перезагрузка" \
            --yesno "Вы внесли изменения в конфигурацию Repka Pi 4\nДля вступления изменений в силу необходимо перезагрузить устройство\nВыполнить перезагрузку?" \
            10 80) then
            echo "Перезагружаем $?."
            reboot
      else
       return 0
      fi
  fi
}

need_reboot() {
  if (
    whiptail \
      --title "Необходима перезагрузка" \
      --yesno "Чтобы изменения вступили в силу необходимо перезагрузить Repka Pi 4\nВыполнить перезагрузку?" \
      10 80) then
        echo "Перезагружаем $?."
        reboot
  else
   return 0
  fi
}

set_console_param() {
  CONSOLE_PARAM=$1
  repka-control set-console "$1"
}

set_kernel_log_level() {
  KERNEL_LOG_LEVEL=$1
  repka-control set-kernel-log-level ${KERNEL_LOG_LEVEL}
}

# =================== 3 Interface Options ===================

set_pinout() {
  PINOUT=$1
  repka-control set-pinout "$1"
}

kernel_log_level() {
  BREADCRUBS=$1
  KL0_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "0" ]] && echo "ON" || echo "OFF")
  KL1_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "1" ]] && echo "ON" || echo "OFF"])
  KL2_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "2" ]] && echo "ON" || echo "OFF")
  KL3_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "3" ]] && echo "ON" || echo "OFF")
  KL4_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "4" ]] && echo "ON" || echo "OFF")
  KL5_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "5" ]] && echo "ON" || echo "OFF")
  KL6_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "6" ]] && echo "ON" || echo "OFF")
  KL7_STATUS=$([[ ${KERNEL_LOG_LEVEL} = "7" ]] && echo "ON" || echo "OFF")

  RESULT=$(
      whiptail \
        --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
        --radiolist \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
        "$BREADCRUBS\n\nЭтот параметр отвечает за уровень логирования ядра Linux. Чем уровень выше, тем больше логов будет.\n\n${RADIOLIST_TIP}" \
        22 100 8 \
        "KL0 | KERN_EMERG" "Аварийные сообщения, система неработоспособна" $KL0_STATUS \
        "KL1 | KERN_ALERT" "Сообщения о критических ошибках, требующих немедленного внимания" $KL1_STATUS \
        "KL2 | KERN_CRIT" "Сообщения о критических ошибках" $KL2_STATUS \
        "KL3 | KERN_ERR" "Сообщения об ошибках" $KL3_STATUS \
        "KL4 | KERN_WARNING" "Предупреждения" $KL4_STATUS \
        "KL5 | KERN_NOTICE" "Уведомления" $KL5_STATUS \
        "KL6 | KERN_INFO" "Информационные сообщения" $KL6_STATUS \
        "KL7 | KERN_DEBUG" "Информационные сообщения" $KL7_STATUS \
        3>&1 1>&2 2>&3)

    RET=$?
    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      debug_log "1 System Options/S2 Debug Log"
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        KL0\ *) set_kernel_log_level 0 && need_reboot ;;
        KL1\ *) set_kernel_log_level 1 & need_reboot ;;
        KL2\ *) set_kernel_log_level 2 && need_reboot ;;
        KL3\ *) set_kernel_log_level 3 && need_reboot ;;
        KL4\ *) set_kernel_log_level 4 && need_reboot ;;
        KL5\ *) set_kernel_log_level 5 && need_reboot ;;
        KL6\ *) set_kernel_log_level 6 && need_reboot ;;
        KL7\ *) set_kernel_log_level 7 && need_reboot ;;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

kernel_output_device() {
  BREADCRUBS=$1
    KOD1_STATUS=$([[ ${CONSOLE_PARAM} = "both" ]] && echo "ON" || echo "OFF")
    KOD2_STATUS=$([[ ${CONSOLE_PARAM} = "display" ]] && echo "ON" || echo "OFF")
    KOD3_STATUS=$([[ ${CONSOLE_PARAM} = "serial" ]] && echo "ON" || echo "OFF")

    RESULT=$(
      whiptail \
        --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
        --radiolist \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
        "$BREADCRUBS\n\nДанный параметр отвечает за выбор устройства, в которое ядро Linux будет выводить сообщения логов.\n\n${RADIOLIST_TIP}" \
        17 80 3 \
        "KOD1 | Дисплей (tty6) и последовательный порт (UART0) " "" $KOD1_STATUS \
        "KOD2 | Дисплей (tty6) " "" $KOD2_STATUS \
        "KOD3 | Последовательный порт (UART0) " "" $KOD3_STATUS \
        3>&1 1>&2 2>&3)

    RET=$?
    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      debug_log "1 System Options/S2 Debug Log"
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        KOD1\ *)
          set_console_param "both"
          need_reboot
          system_options "1 System Options" ;;

        KOD2\ *)
          set_console_param "display"
          need_reboot
          system_options "1 System Options" ;;

        KOD3\ *)
          set_console_param "serial"
          need_reboot
          system_options "1 System Options" ;;

        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

switch_off_boot_message() {
  BOOT_MESSAGE="0"
  repka-control set-boot-message "0"
  dd if=/boot/spl/u-boot-sunxi-with-spl-silent.bin of=/dev/${ROOT_DEVICE} bs=1024 seek=8 conv=notrunc
}

switch_on_boot_message() {
  BOOT_MESSAGE="1"
  repka-control set-boot-message "1"
  dd if=/boot/spl/u-boot-sunxi-with-spl.bin of=/dev/${ROOT_DEVICE} bs=1024 seek=8 conv=notrunc
}

set_interface() {
  repka-control set-custom-port "$1" "$2"
  if [ $PINOUT = "custom" ]; then
      set_pinout "custom" && need_reboot
  fi
}

  select_interface() {
  if [ "$#" -ne 1 ]; then
    echo "Необходимо использовать аргумент CP с номером"
    return 1
  fi

  case "$1" in
    CP1)
      CP1_RESULT=$(repka-control get-custom-port 1)
      CP1_STATUS=$([[ "${CP1_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP1_RESULT}" = "i2c1" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP1 | 3,5 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:\n\n${RADIOLIST_TIP}" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "I2C1" "" ${CP2_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "1" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2C1") set_interface "1" "i2c1" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP2)
      CP2_RESULT=$(repka-control get-custom-port 2)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "spi0" ]] && echo "ON" || echo "OFF")
      CP3_STATUS=$([[ "${CP3_RESULT}" = "i2c3" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP2 | 19, 21, 23, 24, 26 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:\n\n${RADIOLIST_TIP}" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "SPI0" "" ${CP2_STATUS} \
          "I2C3" "" ${CP3_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "2" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "SPI0") set_interface "2" "spi0" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2C3") set_interface "2" "i2c3" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP3)
      CP2_RESULT=$(repka-control get-custom-port 3)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "i2c2" ]] && echo "ON" || echo "OFF")
      CP3_STATUS=$([[ "${CP2_RESULT}" = "uart3" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP3 | 27, 28 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:\n\n${RADIOLIST_TIP}" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "I2C2" "" ${CP2_STATUS} \
          "UART3" "" ${CP3_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "3" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2C2") set_interface "3" "i2c2" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "UART3") set_interface "3" "uart3" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP4)
      CP2_RESULT=$(repka-control get-custom-port 4)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "s_uart" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP3 | 27, 28 Pins" \
          --radiolist \
          "Выберите доступный интерфейс:\n\n${RADIOLIST_TIP}" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "S-UART" "" ${CP2_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "4" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "S-UART") set_interface "4" "s_uart" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP5)
      CP2_RESULT=$(repka-control get-custom-port 5)
      CP1_STATUS=$([[ "${CP2_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP2_RESULT}" = "w1_gpio" ]] && echo "ON" || echo "OFF")

      RESULT=$(
        whiptail \
          --title "CP5 | 7 Pin" \
          --radiolist \
          "Выберите доступный интерфейс:\n\n${RADIOLIST_TIP}" \
          --cancel-button "Назад/Back" \
          --ok-button "Выбрать/Select" \
          20 50 4 \
          "GPIO" "" ${CP1_STATUS} \
          "W1-GPIO" "" ${CP2_STATUS} \
          3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "5" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "W1-GPIO") set_interface "5" "w1_gpio" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
      ;;

    CP6)
      CP6_RESULT=$(repka-control get-custom-port 6)
      CP1_STATUS=$([[ "${CP6_RESULT}" = "" ]] && echo "ON" || echo "OFF")
      CP2_STATUS=$([[ "${CP6_RESULT}" = "i2s" ]] && echo "ON" || echo "OFF")
      CP3_STATUS=$([[ "${CP6_RESULT}" = "i2s_pcm5102" ]] && echo "ON" || echo "OFF")
      CP4_STATUS=$([[ "${CP6_RESULT}" = "i2s_pcm5122" ]] && echo "ON" || echo "OFF")

      RESULT=$(
      whiptail \
        --title "CP6 | 12, 35, 36, 38, 40 Port" \
        --radiolist \
        "Выберите доступный интерфейс:\n\n${RADIOLIST_TIP}" \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
        18 50 6 \
        "GPIO" "" $CP1_STATUS \
        "I2S" "" $CP2_STATUS \
        "I2S-PCM5102" "" $CP3_STATUS \
        "I2S-PCM5122" "" $CP4_STATUS \
        3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        custom_pinout_config "3 Interface Options/I3 Custom Pinout"
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          "GPIO") set_interface "6" "" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2S") set_interface "6" "i2s" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2S-PCM5102") set_interface "6" "i2s_pcm5102" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          "I2S-PCM5122") set_interface "6" "i2s_pcm5122" && custom_pinout_config "3 Interface Options/I3 Custom Pinout" ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
    ;;
    *)
        echo "Invalid argument: $1"
        return 1
        ;;
  esac
}

sd2emmc() {
  nand-sata-install && return
}

boot_messages() {
  BREADCRUBS=$1
  BM1_STATUS=$([[ ${BOOT_MESSAGE} = "1" ]] && echo "ON" || echo "OFF")
  BM2_STATUS=$([[ ${BOOT_MESSAGE} = "0" ]] && echo "ON" || echo "OFF")

  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
      --radiolist \
      --cancel-button "Назад/Back" \
      --ok-button "Выбрать/Select" \
      "$BREADCRUBS\n\nПараметр отвечает за режим вывода сообщений загрузчика U-Boot.\n\n${RADIOLIST_TIP}" \
      16 80 3 \
      "BM1 | Включить вывод сообщений загрузчика U-Boot" "" $BM1_STATUS \
      "BM2 | Отключить вывод сообщений загрузчика U-Boot" "" $BM2_STATUS \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
    debug_log "1 System Options/S2 Debug Log"
  elif [ $RET -eq 0 ]; then
    case "$RESULT" in
      BM1\ *) switch_on_boot_message && need_reboot ;;
      BM2\ *) switch_off_boot_message && need_reboot ;;
      *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
    esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
  fi
}

custom_pinout_config() {
  BREADCRUBS=$1
  CP1="$(repka-control get-custom-port 1)"
  CP2="$(repka-control get-custom-port 2)"
  CP3="$(repka-control get-custom-port 3)"
  CP4="$(repka-control get-custom-port 4)"
  CP5="$(repka-control get-custom-port 5)"
  CP6="$(repka-control get-custom-port 6)"

  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}"  \
      --menu "$BREADCRUBS \n\nВыберите нужный набор пинов для установки конкретного интерфейса:" \
      20 90 10 \
      --cancel-button "Выйти/Finish" \
      --ok-button "Выбрать/Select" \
      "3, 5 Pins" "  - $(if [[ -z "$CP1" ]]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [[ $CP1 = "i2c1" ]]; then echo '[I2C1-Выбран]'; else echo 'I2C1'; fi)" \
      "7 Pin" "  - $(if [[ -z "$CP5" ]]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [[ $CP5 = "w1_gpio" ]]; then echo '[W1-GPIO-Выбран]'; else echo 'W1-GPIO'; fi)" \
      "16, 18 Pins" "  - $(if [[ -z "$CP4" ]]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [[ $CP4 = "s_uart" ]]; then echo '[S-UART-Выбран]'; else echo 'S-UART'; fi)" \
      "19, 21, 23, 24, 26 Pins" "  - $(if [[ -z "$CP2" ]]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [[ $CP2 = "spi0" ]]; then echo '[SPI0-Выбран]'; else echo 'SPI0'; fi)/$(if [[ $CP2 = "i2c3" ]]; then echo '[I2C3-Выбран]'; else echo 'I2C3'; fi)" \
      "27, 28 Pins" "  - $(if [[ -z "$CP3" ]]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [[ $CP3 = "i2c2" ]]; then echo '[I2C2-Выбран]'; else echo 'I2C2'; fi)/$(if [[ $CP3 = "uart3" ]]; then echo '[UART3-Выбран]'; else echo 'UART3'; fi)" \
      "12, 35, 36, 38, 40 Pins" "  - $(if [[ -z "$CP6" ]]; then echo '[GPIO-Выбран]'; else echo 'GPIO'; fi)/$(if [[ $CP6 = "i2s" ]]; then echo '[I2S-Выбран]'; else echo 'I2S'; fi)/$(if [[ $CP6 = "i2s_pcm5102" ]]; then echo '[I2S-PCM5102-Выбран]'; else echo 'I2S-PCM5102'; fi)/$(if [[ $CP6 = "i2s_pcm5122" ]]; then echo '[I2S-PCM5122-Выбран]'; else echo 'I2S-PCM5122'; fi)" \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
    interface_options "3 Interface Options"
  elif [ $RET -eq 0 ]; then
    case "$RESULT" in
      "3, 5 Pins") select_interface "CP1" ;;
      "7 Pin") select_interface "CP5" ;;
      "16, 18 Pins") select_interface "CP4" ;;
      "19, 21, 23, 24, 26 Pins") select_interface "CP2" ;;
      "27, 28 Pins") select_interface "CP3" ;;
      "12, 35, 36, 38, 40 Pins") select_interface "CP6" ;;
      *) whiptail --msgbox "Ошибка" 20 60 1 ;;
    esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1
  fi
}

# =================== 7 Update ===================

repka_os_update() {
  REPKA_OS_UPDATE=/usr/local/repka-os-update
  wget -O $REPKA_OS_UPDATE --timeout=120 https://download.repka-pi.ru/repka-pi4/repka_os_update/repka-os-update
  chmod +x $REPKA_OS_UPDATE
  $REPKA_OS_UPDATE
  rm $REPKA_OS_UPDATE
}

ubuntu_update() {
  apt-get update &&
  apt-get upgrade &&
  printf "Ожидаем 5 секунд до повторного запуска repka-config\n" &&
  sleep 5 &&
  exec repka-config
}

# I2 Pinout Options
pinout_options() {
  BREADCRUBS=$1

  PINOUT_1=$([[ "${PINOUT}" = "1" ]] && echo "ON" || echo "OFF")
  PINOUT_2=$([[ "${PINOUT}" = "2" ]] && echo "ON" || echo "OFF")
  PINOUT_3=$([[ "${PINOUT}" = "3" ]] && echo "ON" || echo "OFF")
  PINOUT_4=$([[ "${PINOUT}" = "4" ]] && echo "ON" || echo "OFF")
  PINOUT_7=$([[ "${PINOUT}" = "7" ]] && echo "ON" || echo "OFF")
  PINOUT_8=$([[ "${PINOUT}" = "8" ]] && echo "ON" || echo "OFF")
  PINOUT_9=$([[ "${PINOUT}" = "9" ]] && echo "ON" || echo "OFF")
  PINOUT_11=$([[ "${PINOUT}" = "11" ]] && echo "ON" || echo "OFF")
  PINOUT_CUSTOM=$([[ "${PINOUT}" = "custom" ]] && echo "ON" || echo "OFF")

  OPTION=$(
    whiptail \
      --title "Выбор распиновки" \
      --radiolist "$BREADCRUBS \n\nВыберите вариант распиновки портов Repka Pi:\n\n${RADIOLIST_TIP}" \
      23 85 10 \
      --cancel-button "Назад/Back" \
      --ok-button "Выбрать/Select"\
      "1 Вариант" "  - GPIO-24, UART-1, PWM-1 (По умолчанию) " $PINOUT_1 \
      "2 Вариант" "  - GPIO-18, UART-1, SPI-1, I2C-1, PWM-1" $PINOUT_2 \
      "3 Вариант" "  - GPIO-16, UART-1, SPI-1, I2C-2, PWM-1" $PINOUT_3 \
      "4 Вариант" "  - GPIO-14, UART-2, S-UART-1, SPI-1, I2C-1, PWM-1" $PINOUT_4 \
      "7 Вариант" "  - GPIO-20, UART-1, SPI-1, I2C-1, I2S " $PINOUT_7 \
      "8 Вариант" "  - GPIO-20, UART-1, SPI-1, I2C-1, I2S-PCM5102 " $PINOUT_8 \
      "9 Вариант" "  - GPIO-20, UART-1, SPI-1, I2C-1, I2S-PCM5122 " $PINOUT_9 \
      "11 Вариант" "  - GPIO-22, UART-1, I2C-1, W1_GPIO-1" $PINOUT_11 \
      "12 Свой вариант" " - Выберите необходимые для вас интерфейсы" $PINOUT_CUSTOM \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
    interface_options "3 Interface Options"
  elif [ $RET -eq 0 ]; then
    case "$OPTION" in
  1\ *) set_pinout 1 && need_reboot ;;
  2\ *) set_pinout 2 && need_reboot ;;
  3\ *) set_pinout 3 && need_reboot ;;
  4\ *) set_pinout 4 && need_reboot ;;
  7\ *) set_pinout 7 && need_reboot ;;
  8\ *) set_pinout 8 && need_reboot ;;
  9\ *) set_pinout 9 && need_reboot ;;
  11\ *) set_pinout 11 && need_reboot ;;
  12\ *) set_pinout "custom" && need_reboot ;;
      *) whiptail --msgbox "Ошибка" 20 60 1 ;;
    esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1
  fi
}


debug_log() {
  BREADCRUBS="$1/"
  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
      --menu "$BREADCRUBS" \
      20 80 10 \
      --cancel-button "Назад/Back"\
      --ok-button "Выбрать/Select" \
      "DL1 Kernel Output Device" "Устройство для вывода сообщений ядра Linux" \
      "DL2 Kernel Log Level " "Уровень логирования ядра Linux" \
      "DL3 Boot Message " "Выводить ли сообщения загрузчика U-Boot" \
      3>&1 1>&2 2>&3)

    RET=$?

    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      system_options "1 System Options"
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        DL1\ *) kernel_output_device "${BREADCRUBS}DL1 Kernel Output Device" ;;
        DL2\ *) kernel_log_level "${BREADCRUBS}DL2 Kernel Log Level" ;;
        DL3\ *) boot_messages "${BREADCRUBS}DL3 Boot Message" ;;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

# 1 System Options
system_options() {
  BREADCRUBS="$1/"
  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
      --menu "$BREADCRUBS" \
      20 80 10 \
      --cancel-button "Назад/Back"\
      --ok-button "Выбрать/Select" \
  	  "S1 OS Copying " "Копирование ОС с одного накопителя на другой" \
  	  "S2 Debug Log" "Отладочные сообщения Repka Pi" \
      3>&1 1>&2 2>&3)

    RET=$?

    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        S1\ *) sd2emmc ;;
        S2\ *) debug_log "${BREADCRUBS}S2 Debug Log" ;;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

# 3 Interface Options
interface_options() {
  BREADCRUBS="$1/"
  RESULT=$(
    whiptail \
    --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
    --menu "$BREADCRUBS" \
    20 80 10 \
    --cancel-button "Назад/Back" \
    --ok-button "Выбрать/Select" \
  	"I1 Pinout Options" "Выбор распиновки" \
  	"I2 Custom Pinout" "Настройка своего варианта распиновки" \
    3>&1 1>&2 2>&3)

    RET=$?

    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        I1\ *) pinout_options "${BREADCRUBS}I1 Pinout Options" ;;
        I2\ *) custom_pinout_config "${BREADCRUBS}I2 Custom Pinout" ;;
        *) whiptail --msgbox "В разработке $RESULT" 20 60 1 ;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}


hardware_acceleration_turn_on() {
  apt install -y ffmpeg-v4l2request mpv \
  && echo -e "hwdec=drm\ndrm-drmprime-video-plane=primary\ndrm-draw-plane=overlay" | tee /etc/mpv/mpv.conf \
  && echo 1 > /etc/repka-wallpaper/hardware-acceleration \
  && repka-wallpaper \
  && need_reboot
}

hardware_acceleration_turn_off() {
  apt purge -y ffmpeg-v4l2request mpv \
    && apt autoremove -y \
    && echo 0 > /etc/repka-wallpaper/hardware-acceleration \
    && repka-wallpaper \
    && need_reboot
}

hardware_acceleration() {
    BREADCRUBS="$1/"
    RESULT=$(
      whiptail \
        --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
        --menu "$BREADCRUBS" \
        20 80 10 \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
    	  "Turn-On " "Включить аппаратное ускорение" \
    	  "Turn-Off " "Выключить аппаратное ускорение" \
        3>&1 1>&2 2>&3)

      RET=$?
      if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
        return 0
      elif [ $RET -eq 0 ]; then
        case "$RESULT" in
          Turn-On\ *) hardware_acceleration_turn_on ;;
          Turn-Off\ *) hardware_acceleration_turn_off ;;
          *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
        esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
      fi
}

# 4 Performance Options
performance_options() {
  BREADCRUBS="$1/"
  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
      --menu "$BREADCRUBS" \
      20 80 10 \
      --cancel-button "Назад/Back" \
      --ok-button "Выбрать/Select" \
  	  "P1 Hardware Acceleration " "Аппаратное ускорение" \
      3>&1 1>&2 2>&3)

    RET=$?
    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        P1\ *) hardware_acceleration "${BREADCRUBS}P1 Hardware Acceleration";;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
}

# 7 Update
update() {
  BREADCRUBS="$1/"
  echo " А есть ли интернет....хм... ? "
  if ping -c 1 repka-pi.ru > /dev/null 2>&1; then
    echo " ура интернет ^___^ "

    RESULT=$(
      whiptail \
        --title "Repka Pi утилита для настройки (repka-config) ${VERSION}" \
        --menu "$BREADCRUBS" \
        20 80 10 \
        --cancel-button "Назад/Back" \
        --ok-button "Выбрать/Select" \
        "U1 Ubuntu update pkg " "Обновить пакеты Ubuntu" \
        "U2 Repka OS update " "Обновить файлы Repka-OS" \
        3>&1 1>&2 2>&3)

    RET=$?

    if [ $RET -eq 1 ] || [ $RET -eq 255 ]; then
      return 0
    elif [ $RET -eq 0 ]; then
      case "$RESULT" in
        U1\ *) ubuntu_update ;;
        U2\ *) repka_os_update ;;
        *) whiptail --msgbox "Ошибка" 20 60 1 || return 0;;
      esac || whiptail --msgbox "Что то тут не так $RESULT" 20 60 1 || return 0
    fi
  else
    whiptail --msgbox "Отсутствует подключение к интернету  " 10 40 1 || return 0
  fi
}

# 10 Repka Pi Community
repka_community() {
  nohup telegram-desktop -- tg://resolve?domain=RepkaPitalk > /dev/null 2>&1 &

  whiptail \
    --title "10 Repka Pi Community" \
    --msgbox "В отдельном окне откроется Telegram с чатом нашего сообщества.\n\nВ нем можно задать любой интересующий вас вопрос." \
    10 60

  return 0
}

# Основной цикл утилиты настройки с главным меню
while true; do
  RESULT=$(
    whiptail \
      --title "Repka Pi утилита для настройки (repka-config) ${CONFIG_VERSION}"  \
      --menu "Выберите настраиваемую опцию. (Для выхода из repka-config нажмите ESC)" \
      20 80 10 \
      --cancel-button "Выйти/Finish" \
      --ok-button "Выбрать/Select" \
      "1 System Options" "Системные настройки ОС" \
      "2 Display Options" "Настройка параметров дисплея" \
      "3 Interface Options" "Настройка подключения интерфейсов" \
      "4 Performance Options" "Настройка производительности" \
      "5 Localization Options" "Настройка региона и языка" \
      "6 Advanced Options" "Продвинутые настройки Repka OS" \
      "7 Update" "Обновление компонентов Repka OS" \
      "8 Board Options" "Параметры платы Repka Pi" \
      "9 About repka-config" "Информация по утилите настройки" \
      3>&1 1>&2 2>&3)

  RET=$?
  if [ $RET -eq 1 ]; then
    exit
    return 0
  elif [ $RET -eq 0 ]; then
    case "$RESULT" in
      1\ *) system_options "1 System Options" ;;
      3\ *) interface_options "3 Interface Options" ;;
      4\ *) performance_options "4 Performance Options" ;;
      7\ *) update "7 Update" ;;
      *) whiptail --msgbox "В разработке $RESULT" 20 60 1 ;;
    esac || whiptail --msgbox "В разработке $RESULT" 20 60 1
  else
    check_need_reboot
    exit 1
  fi
done